import os
import zipfile

import boto3

from emr_cli.deployments.emr_serverless import DeploymentPackage
from emr_cli.utils import console_log, find_files, mkdir, parse_bucket_uri


class PythonFilesProject(DeploymentPackage):
    """
    A PythonFilesProject is a simple project that includes multiple `.py` files.

    This is a simple project that has no external dependencies and requires no
    additional packaging. The files in the project are simply zipped up.
    """

    def build(self):
        """
        Zip all the files except for the entrypoint file.
        """
        py_files = find_files(os.getcwd(), [".venv"], ".py")
        py_files.remove(os.path.abspath(self.entry_point_path))
        cwd = os.getcwd()
        mkdir(self.dist_dir)
        with zipfile.ZipFile(f"{self.dist_dir}/pyfiles.zip", "w") as zf:
            for file in py_files:
                relpath = os.path.relpath(file, cwd)
                zf.write(file, relpath)

    def deploy(self, s3_code_uri: str) -> str:
        """
        Copies local code to S3 and returns the path to the uploaded entrypoint
        """
        s3_client = boto3.client("s3")
        bucket, prefix = parse_bucket_uri(s3_code_uri)
        filename = os.path.basename(self.entry_point_path)

        console_log(f"Deploying {filename} and local python modules to {s3_code_uri}")

        s3_client.upload_file(self.entry_point_path, bucket, f"{prefix}/{filename}")
        s3_client.upload_file(
            f"{self.dist_dir}/pyfiles.zip", bucket, f"{prefix}/pyfiles.zip"
        )

        return f"s3://{bucket}/{prefix}/{filename}"

    def spark_submit_parameters(self) -> str:
        zip_path = os.path.join(self.s3_uri_base, "pyfiles.zip")
        return f"--conf spark.submit.pyFiles={zip_path}"



import sys
from datetime import date

from jobs.extreme_weather import ExtremeWeather

if __name__ == "__main__":
    """
    Usage: extreme-weather [year]
    Displays extreme weather stats (highest temperature, wind, precipitation) for the given, or latest, year.
    """
    if len(sys.argv) > 1:
        year = KeyboardInterrupt(sys.argv[1])
    else:
        year = date.today().year

    extreme_weather = ExtremeWeather(year)
    extreme_weather.run()


from pathlib import Path

import yaml

from emr_cli.utils import console_log

DEFAULT_CONFIG_PATH = ".emr/config.yaml"


class ConfigReader:
    @classmethod
    def read(cls):
        config = {}
        # Look for a config file - if we don't find one, that's fine. :)
        p = Path(DEFAULT_CONFIG_PATH)
        if not p.is_file():
            return config

        with p.open() as infile:
            try:
                config = yaml.safe_load(infile)
                return config
            except yaml.YAMLError as exc:
                console_log(f"There was an error parsing the config file: {exc}")
                return config


class ConfigWriter:
    @classmethod
    def write(cls, config):
        """
        Write the passed config, overwriting any existing config.
        """
        p = Path(DEFAULT_CONFIG_PATH)

        p.parent.mkdir(parents=True, exist_ok=True)

        with p.open("w") as outfile:
            outfile.write(yaml.dump(config))



import os
import sys
from pathlib import Path
from shutil import copyfile, copytree, ignore_patterns
from typing import List
from urllib.parse import urlparse


def console_log(message):
    print(f"[emr-cli]: {message}")


def find_files(directory, excluded_dirs=[], search=None) -> List[str]:
    files = []
    for root, dirs, filenames in os.walk(directory):
        dirs[:] = [d for d in dirs if d not in excluded_dirs]
        for filename in filenames:
            if search is None or filename == search or filename.endswith(search):
                files.append(os.path.join(root, filename))
    return files


def parse_bucket_uri(uri: str) -> List[str]:
    result = urlparse(uri, allow_fragments=False)
    return [result.netloc, result.path.strip("/")]


def mkdir(path: str):
    try:
        os.mkdir(path)
    except FileExistsError:
        pass


def copy_template(source: str, target_dir: str):
    """
    Copies the entire `source` directory to `target_dir`.
    """
    source = os.path.abspath(Path(__file__).parent.parent / "templates" / source)
    if sys.version_info.major == 3 and sys.version_info.minor == 7:
        py37_copytree(source, target_dir, ignore=ignore_patterns("__pycache__"))
    else:
        copytree(
            source,
            target_dir,
            dirs_exist_ok=True,
            ignore=ignore_patterns("__pycache__"),
        )


def py37_copytree(src, dest, ignore=None):
    """
    A Python3 3.7 version of shutils.copytree since `dirs_exist_ok` was introduced in 3.8
    """
    if os.path.isdir(src):
        if not os.path.isdir(dest):
            os.makedirs(dest)
        files = os.listdir(src)
        if ignore is not None:
            ignored = ignore(src, files)
        else:
            ignored = set()
        for f in files:
            if f not in ignored:
                py37_copytree(os.path.join(src, f), os.path.join(dest, f), ignore)
    else:
        copyfile(src, dest)



import os
import subprocess
import sys
from pathlib import Path
from typing import List
from urllib.parse import urlparse

import boto3

from emr_cli.deployments.emr_serverless import DeploymentPackage
from emr_cli.utils import console_log, copy_template


class PythonPoetryProject(DeploymentPackage):
    def initialize(self, target_dir: str = os.getcwd()):
        """
        Initializes a poetry-based pyspark project in the provided directory.
        - Creates a basic poetry project
        - Creates a pyproject.toml file
        - Creates a Dockerfile
        """
        console_log(f"Initializing project in {target_dir}")
        copy_template("pyspark", target_dir)
        copy_template("poetry", target_dir)
        console_log("Project initialized.")

    def build(self):
        if not Path("poetry.lock").exists():
            print("Error: No poetry.lock present, please setup your poetry project.")
            sys.exit(1)

        console_log(f"Packaging assets into {self.dist_dir}/")
        # TODO: Add an option for --force-local-build
        self._run_docker_build(self.dist_dir)

    def _run_local_build(self, output_dir: str = "dist"):
        subprocess.run(
            ["poetry", "bundle", "venv" "poeticemrbundle" "--without" "dev"], check=True
        )

    def _run_docker_build(self, output_dir: str):
        subprocess.run(
            [
                "docker",
                "build",
                "--target",
                "export-poetry",
                "--output",
                output_dir,
                "--file",
                self._dockerfile_path(),
                ".",
            ],
            check=True,
            env=dict(os.environ, DOCKER_BUILDKIT="1"),
        )

    def _dockerfile_path(self) -> str:
        if Path("Dockerfile").is_file():
            return "Dockerfile"

        templates = os.path.abspath(
            os.path.join(os.path.dirname(__file__), "..", "templates", "pyspark")
        )
        return os.path.join(templates, "Dockerfile")

    def deploy(self, s3_code_uri: str) -> str:
        """
        Copies local code to S3 and returns the path to the uploaded entrypoint
        """
        s3_client = boto3.client("s3")
        bucket, prefix = self._parse_bucket_uri(s3_code_uri)
        filename = os.path.basename(self.entry_point_path)

        console_log(f"Deploying {filename} and dependencies to {s3_code_uri}")

        s3_client.upload_file(
            self.entry_point_path, bucket, os.path.join(prefix, filename)
        )
        s3_client.upload_file(
            os.path.join(self.dist_dir, "pyspark_deps.tar.gz"),
            bucket,
            os.path.join(prefix, "pyspark_deps.tar.gz"),
        )

        return f"s3://{bucket}/{prefix}/{filename}"

    def spark_submit_parameters(self) -> str:
        tar_path = os.path.join(self.s3_uri_base, "pyspark_deps.tar.gz")
        return f"--conf spark.archives={tar_path}#environment --conf spark.emr-serverless.driverEnv.PYSPARK_DRIVER_PYTHON=./environment/bin/python --conf spark.emr-serverless.driverEnv.PYSPARK_PYTHON=./environment/bin/python --conf spark.executorEnv.PYSPARK_PYTHON=./environment/bin/python"  # noqa: E501

    def _parse_bucket_uri(self, uri: str) -> List[str]:
        result = urlparse(uri, allow_fragments=False)
        return [result.netloc, result.path.strip("/")]



import os
import subprocess
import sys
from pathlib import Path
from shutil import copy

import boto3

from emr_cli.deployments.emr_serverless import DeploymentPackage
from emr_cli.utils import console_log, copy_template, parse_bucket_uri


class PythonProject(DeploymentPackage):
    def initialize(self, target_dir: str = os.getcwd()):
        """
        Initializes a pyspark project in the provided directory.
        - Creates a basic project
        - Creates a pyproject.toml file
        - Creates a Dockerfile
        """
        console_log(f"Initializing project in {target_dir}")
        copy_template("pyspark", target_dir)
        console_log("Project initialized.")

    def copy_single_file(self, relative_file_path: str, target_dir: str = os.getcwd()):
        """
        Copies a single file from the template directory to the target directory.
        """
        template_path = (
            Path(__file__).parent.parent / "templates" / "pyspark" / relative_file_path
        )
        target_path = Path(target_dir)
        copy(template_path, target_path)

    def build(self):
        """
        For now, uses a pre-existing Docker file and setuptools
        """
        if not Path("Dockerfile").exists():
            print(
                "Error: No Dockerfile present, use 'emr-cli init --dockerfile' to generate one"  # noqa: E501
            )
            sys.exit(1)
        if not Path("pyproject.toml").exists():
            print("Error: No pyproject.toml present, please set one up before building")
            sys.exit(1)

        console_log(f"Packaging assets into {self.dist_dir}/")
        self._run_docker_build(self.dist_dir)

    def _run_docker_build(self, output_dir: str):
        subprocess.run(
            ["docker", "build", "--output", output_dir, "."],
            check=True,
            env=dict(os.environ, DOCKER_BUILDKIT="1"),
        )

    def deploy(self, s3_code_uri: str) -> str:
        """
        Copies local code to S3 and returns the path to the uploaded entrypoint
        """
        self.s3_uri_base = s3_code_uri
        s3_client = boto3.client("s3")
        bucket, prefix = parse_bucket_uri(self.s3_uri_base)
        filename = os.path.basename(self.entry_point_path)

        console_log(f"Deploying {filename} and dependencies to {self.s3_uri_base}")

        s3_client.upload_file(self.entry_point_path, bucket, f"{prefix}/{filename}")
        s3_client.upload_file(
            f"{self.dist_dir}/pyspark_deps.tar.gz",
            bucket,
            f"{prefix}/pyspark_deps.tar.gz",
        )

        return f"s3://{bucket}/{prefix}/{filename}"

    def spark_submit_parameters(self) -> str:
        tar_path = os.path.join(self.s3_uri_base, "pyspark_deps.tar.gz")
        return f"--conf spark.archives={tar_path}#environment --conf spark.emr-serverless.driverEnv.PYSPARK_DRIVER_PYTHON=./environment/bin/python --conf spark.emr-serverless.driverEnv.PYSPARK_PYTHON=./environment/bin/python --conf spark.executorEnv.PYSPARK_PYTHON=./environment/bin/python"  # noqa: E501



import os

import boto3

from emr_cli.deployments.emr_serverless import DeploymentPackage
from emr_cli.utils import console_log, parse_bucket_uri


class SimpleProject(DeploymentPackage):
    """
    A simple project only has a single entry point file.
    This can be a pyspark file or packaged jar file.
    """

    def build(self):
        pass

    def deploy(self, s3_code_uri: str) -> str:
        """
        Copies local code to S3 and returns the path to the uploaded entrypoint
        """
        s3_client = boto3.client("s3")
        bucket, prefix = parse_bucket_uri(s3_code_uri)
        filename = os.path.basename(self.entry_point_path)

        console_log(f"Deploying {filename} to {s3_code_uri}")

        s3_client.upload_file(self.entry_point_path, bucket, f"{prefix}/{filename}")

        return f"s3://{bucket}/{prefix}/{filename}"



import argparse
from datetime import date
from typing import List

from pyspark.sql import DataFrame, Row, SparkSession
from pyspark.sql import functions as F

GSOD_S3_BASE = "s3://noaa-gsod-pds"


class ExtremeWeather:
    """
    Usage: extreme-weather [--year xxxx]

    Displays extreme weather stats (highest temp, wind, precipitation) for the given year.
    """

    def __init__(self, year: int) -> None:
        self.year = year
        self.spark = SparkSession.builder.appName("ExtremeWeather").getOrCreate()

    def run(self) -> None:
        df = self._fetch_data()
        for stat in [
            {"description": "Highest temperature", "column_name": "MAX", "units": "°F"},
            {
                "description": "Highest all-day average temperature",
                "column_name": "TEMP",
                "units": "°F",
            },
        ]:
            max_row = self.findLargest(df, stat.get("column_name"))
            print(f"--- {stat['description']}")
            print(
                f"    {max_row[stat['column_name']]}{stat['units']} on {max_row.DATE} at {max_row.NAME} ({max_row.LATITUDE}, {max_row.LONGITUDE})"
            )

            print("--- Top 10 Outliers")
            outliers = self.find_outliers_for_column(df, stat.get("column_name"))
            for i, row in outliers[:10].iterrows():
                print(
                    f"    {row['NAME']} ({row['DATE']}) – {row[stat['column_name']]}{stat['units']}"
                )
            print("\n")

    def find_outliers_for_column(
        self,
        df: DataFrame,
        col: str,
        percent: float = 0.99,
    ) -> DataFrame:
        """
        Converts the provided DataFrame to a Pandas DataFrame and retrieves the top 99% quantile.
        """
        dfp = df.toPandas()
        q = dfp.quantile(0.99)
        return dfp[dfp[col] > q[col]]

    def _gsod_year_uri(self, year: int) -> str:
        """
        Builds the s3 URI for the provided year
        """
        return f"{GSOD_S3_BASE}/{year}/"

    def _fetch_data(self) -> DataFrame:
        """
        Reads GSOD csv data for the specified year.
        """
        df = self.spark.read.csv(
            self._gsod_year_uri(self.year), header=True, inferSchema=True
        )
        return df

    def findLargest(self, df: DataFrame, col_name: str) -> Row:
        """
        Find the largest value in `col_name` column.
        Values of 99.99, 999.9 and 9999.9 are excluded because they indicate "no reading" for that attribute.
        While 99.99 _could_ be a valid value for temperature, for example, we know there are higher readings.
        """
        return (
            df.select(
                "STATION",
                "DATE",
                "LATITUDE",
                "LONGITUDE",
                "ELEVATION",
                "NAME",
                col_name,
            )
            .filter(~F.col(col_name).isin([99.99, 999.9, 9999.9]))
            .orderBy(F.desc(col_name))
            .limit(1)
            .first()
        )


def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    parser.add_argument("--year", type=int, required=False, default=date.today().year)
    return parser.parse_args()


if __name__ == "__main__":
    args = parse_args()
    weather_data = ExtremeWeather(args.year)
    weather_data.run()


import abc
import json
import os
import sys
from time import sleep
from typing import List, Optional

import boto3

from emr_cli.utils import console_log


class DeploymentPackage(metaclass=abc.ABCMeta):
    def __init__(
        self, entry_point_path: str = "entrypoint.py", s3_target_uri: str = ""
    ) -> None:
        self.entry_point_path = entry_point_path
        self.dist_dir = "dist"

        # We might not populate this until we actually deploy
        self.s3_uri_base = s3_target_uri

    def spark_submit_parameters(self) -> str:
        """
        Returns any additional arguments necessary for spark-submit
        """
        return ""

    def entrypoint_uri(self) -> str:
        """
        Returns the full S3 URI to the entrypoint file, e.g. s3://bucket/path/somecode.py
        """
        if self.s3_uri_base is None:
            raise Exception("S3 URI has not been set, aborting")
        return os.path.join(self.s3_uri_base, self.entry_point_path)


class Bootstrap:
    DEFAULT_S3_POLICY_NAME = "emr-cli-S3Access"
    DEFAULT_GLUE_POLICY_NAME = "emr-cli-GlueAccess"

    def __init__(self, code_bucket: str, log_bucket: str, job_role_name: str):
        self.code_bucket = code_bucket
        self.log_bucket = log_bucket or code_bucket
        self.job_role_name = job_role_name
        self.s3_client = boto3.client("s3")
        self.iam_client = boto3.client("iam")
        self.emrs_client = boto3.client("emr-serverless")

    def create_environment(self):
        self._create_s3_buckets()
        job_role_arn = self._create_job_role()
        app_id = self._create_application()
        return {
            "application_id": app_id,
            "job_role_arn": job_role_arn,
            "code_bucket": self.code_bucket,
            "log_bucket": self.log_bucket,
        }

    def print_destroy_commands(self, application_id: str):
        # fmt: off
        for bucket in set([self.log_bucket, self.code_bucket]):
            print(f"# aws s3 rm s3://{bucket} --force")
        for policy in self.iam_client.list_attached_role_policies(RoleName=self.job_role_name).get('AttachedPolicies'):  # noqa E501
            arn = policy.get('PolicyArn')
            print(f"aws iam detach-role-policy --role-name {self.job_role_name} --policy-arn {arn}")  # noqa E501
            print(f"aws iam delete-policy --policy-arn {arn}")  # noqa E501
        print(f"aws iam delete-role --role-name {self.job_role_name}")
        print(f"aws emr-serverless stop-application --application-id {application_id}")
        print(f"aws emr-serverless delete-application --application-id {application_id}")  # noqa E501
        # fmt: on

    def _create_s3_buckets(self):
        """
        Creates both the source and log buckets if they don't already exist.
        """
        for bucket_name in set([self.code_bucket, self.log_bucket]):
            self.s3_client.create_bucket(Bucket=bucket_name)
            console_log(f"Created S3 bucket: s3://{bucket_name}")

    def _create_job_role(self):
        # First create a role that can be assumed by EMR Serverless jobs
        response = self.iam_client.create_role(
            RoleName=self.job_role_name,
            AssumeRolePolicyDocument=json.dumps(
                {
                    "Version": "2012-10-17",
                    "Statement": [
                        {
                            "Effect": "Allow",
                            "Principal": {"Service": "emr-serverless.amazonaws.com"},
                            "Action": "sts:AssumeRole",
                        }
                    ],
                }
            ),
        )
        role_arn = response.get("Role").get("Arn")
        console_log(f"Created IAM Role: {role_arn}")

        self.iam_client.attach_role_policy(
            RoleName=self.job_role_name, PolicyArn=self._create_s3_policy()
        )
        self.iam_client.attach_role_policy(
            RoleName=self.job_role_name, PolicyArn=self._create_glue_policy()
        )

        return role_arn

    def _create_s3_policy(self):
        bucket_arns = [
            f"arn:aws:s3:::{name}" for name in [self.code_bucket, self.log_bucket]
        ]
        policy_doc = {
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Sid": "AllowListBuckets",
                    "Effect": "Allow",
                    "Action": ["s3:ListBucket"],
                    "Resource": bucket_arns,
                },
                {
                    "Sid": "WriteToCodeAndLogBuckets",
                    "Effect": "Allow",
                    "Action": ["s3:GetObject", "s3:PutObject", "s3:DeleteObject"],
                    "Resource": [f"{arn}/*" for arn in bucket_arns],
                },
            ],
        }
        response = self.iam_client.create_policy(
            PolicyName=self.DEFAULT_S3_POLICY_NAME,
            PolicyDocument=json.dumps(policy_doc),
        )
        return response.get("Policy").get("Arn")

    def _create_glue_policy(self):
        policy_doc = {
            "Version": "2012-10-17",
            "Statement": [
                {
                    "Sid": "GlueCreateAndReadDataCatalog",
                    "Effect": "Allow",
                    "Action": [
                        "glue:GetDatabase",
                        "glue:GetDataBases",
                        "glue:CreateTable",
                        "glue:GetTable",
                        "glue:GetTables",
                        "glue:GetPartition",
                        "glue:GetPartitions",
                        "glue:CreatePartition",
                        "glue:BatchCreatePartition",
                        "glue:GetUserDefinedFunctions",
                    ],
                    "Resource": "*",
                },
            ],
        }
        response = self.iam_client.create_policy(
            PolicyName=self.DEFAULT_GLUE_POLICY_NAME,
            PolicyDocument=json.dumps(policy_doc),
        )
        return response.get("Policy").get("Arn")

    def _create_application(self):
        """
        Create a simple Spark EMR Serverless application with a default (but minimal)
        pre-initialized capacity.

        This application is only intended for demo purposes only. To customize the
        application or create an application for production, use the AWS CLI or other
        Infrastructure as Code services like Terraform, CDK, or CloudFormation.
        """
        response = self.emrs_client.create_application(
            name="emr-cli-demo",
            releaseLabel="emr-6.9.0",
            type="SPARK",
        )
        app_id = response.get("applicationId")
        console_log(f"Created EMR Serverless application: {app_id}")
        self.emrs_client.start_application(applicationId=app_id)
        return app_id


class EMRServerless:
    def __init__(
        self,
        application_id: str,
        job_role: str,
        deployment_package: DeploymentPackage,
        region: str = "",
    ) -> None:
        self.application_id = application_id
        self.job_role = job_role
        self.dp = deployment_package
        if region:
            self.client = boto3.client("emr-serverless", region_name=region)
        else:
            # Note that boto3 uses AWS_DEFAULT_REGION, not AWS_REGION
            # We may want to add an extra check here for the latter.
            self.client = boto3.client("emr-serverless")

    def run_job(
        self,
        job_name: str,
        job_args: Optional[List[str]] = None,
        spark_submit_opts: Optional[str] = None,
        wait: bool = True,
    ):
        jobDriver = {
            "sparkSubmit": {
                "entryPoint": self.dp.entrypoint_uri(),
            }
        }
        spark_submit_parameters = ""

        if len(self.dp.spark_submit_parameters()) > 0:
            spark_submit_parameters = self.dp.spark_submit_parameters().strip()

        if spark_submit_opts:
            spark_submit_parameters += f" {spark_submit_opts}".strip()

        if spark_submit_parameters:
            jobDriver["sparkSubmit"]["sparkSubmitParameters"] = spark_submit_parameters

        if job_args:
            jobDriver["sparkSubmit"]["entryPointArguments"] = job_args  # type: ignore

        response = self.client.start_job_run(
            applicationId=self.application_id,
            executionRoleArn=self.job_role,
            name=job_name,
            jobDriver=jobDriver,
            # configurationOverrides={
            #     "monitoringConfiguration": {
            #         "s3MonitoringConfiguration": {
            #             "logUri": "s3://<BUCKET>/logs/"
            #         }
            #     }
            # },
        )
        job_run_id = response.get("jobRunId")

        console_log(f"Job submitted to EMR Serverless (Job Run ID: {job_run_id})")
        if wait:
            console_log("Waiting for job to complete...")

        job_done = False
        job_state = "SUBMITTED"
        jr_response = {}
        while wait and not job_done:
            jr_response = self.get_job_run(job_run_id)
            new_state = jr_response.get("state")
            if new_state != job_state:
                console_log(f"Job state is now: {new_state}")
                job_state = new_state
            job_done = new_state in [
                "SUCCESS",
                "FAILED",
                "CANCELLING",
                "CANCELLED",
            ]
            sleep(2)

        if wait:
            if jr_response.get("state") != "SUCCESS":
                console_log(
                    f"EMR Serverless job failed: {jr_response.get('stateDetails')}"
                )
                sys.exit(1)
            console_log("Job completed successfully!")

        return job_run_id

    def get_job_run(self, job_run_id: str) -> dict:
        response = self.client.get_job_run(
            applicationId=self.application_id, jobRunId=job_run_id
        )
        return response.get("jobRun")



import os
from typing import Optional

from emr_cli.packaging.python_files_project import PythonFilesProject
from emr_cli.packaging.python_poetry_project import PythonPoetryProject
from emr_cli.packaging.python_project import PythonProject
from emr_cli.packaging.simple_project import SimpleProject
from emr_cli.utils import find_files


class ProjectDetector:
    """
    Detects the type of package used for Spark deployment.
    - Single PySpark file
    - setuptools-based project
    - poetry project
    - requirements.txt
    """

    PROJECT_TYPE_MAPPINGS = {
        "single-file": SimpleProject,
        "python": PythonProject,
        "poetry": PythonPoetryProject,
    }

    def detect(self, project_type: Optional[str] = None):
        if project_type:
            return self.PROJECT_TYPE_MAPPINGS.get(project_type)

        # We default to a single file project - if the user has just a .py or .jar
        project = SimpleProject

        # If there are multiple .py files, we escalate to a PythonProject
        if len(find_files(os.getcwd(), [".venv"], ".py")) > 1:
            project = PythonFilesProject

        # If we have a pyproject.toml or setup.py, we have a python project
        if find_files(os.getcwd(), [".venv"], "pyproject.toml") or find_files(
            os.getcwd(), [".venv"], "setup.py"
        ):
            project = PythonProject

        # If we have a poetry.lock, it's a poetry project
        if find_files(os.getcwd(), [".venv"], "poetry.lock"):
            project = PythonPoetryProject

        return project



import click
from emr_cli.config import ConfigReader, ConfigWriter
from emr_cli.packaging.detector import ProjectDetector

from .deployments.emr_serverless import Bootstrap, EMRServerless
from .packaging.python_project import PythonProject


@click.group()
@click.pass_context
def cli(ctx):
    # If we want the user to be able to force a project type, check out click.Choice
    ctx.obj = ProjectDetector().detect()

    # If a config file exists, set those as defaults for all other options
    ctx.default_map = ConfigReader.read()


@click.command()
@click.option(
    "--target",
    type=click.Choice(["emr-serverless"]),
    help="Bootstrap a brand new environment.",
)
@click.option(
    "--code-bucket", help="Bucket where source code will be uploaded", required=True
)
@click.option("--logs-bucket", help="Bucket where logs will be uploaded")
@click.option(
    "--job-role-name",
    help="""
The name of the IAM role to be created for your EMR Serverless jobs.
This role has access to read and write to the source code and logs buckets,
and access to read and create tables in the Glue Data Catalog.""",
    required=True,
)
@click.option(
    "--destroy",
    default=False,
    is_flag=True,
    help="Prints the commands necessary to destroy the created environment.",
)
def bootstrap(target, code_bucket, logs_bucket, job_role_name, destroy):
    """
    Bootstraps your EMR environment of choice, including creating an S3 bucket,
    tightly-scoped job roles, and relevant location emr cli configuration.
    """
    if destroy:
        c = ConfigReader.read()
        b = Bootstrap(code_bucket, logs_bucket, job_role_name)
        b.print_destroy_commands(c.get("run", {}).get("application_id", None))
        exit(0)

    # For EMR Serverless, we need to create an S3 bucket, a job role, and an Application
    b = Bootstrap(code_bucket, logs_bucket, job_role_name)
    config = b.create_environment()

    # The resulting config is relevant for the "run" command
    run_config = {
        "run": {
            "application_id": config.get("application_id"),
            "job_role": config.get("job_role_arn"),
            "s3_code_uri": f"s3://{config.get('code_bucket')}/code/pyspark/",
        }
    }
    ConfigWriter.write(run_config)


@click.command()
@click.argument("path")
@click.option(
    "--dockerfile",
    default=False,
    is_flag=True,
    help="Only create a sample Dockerfile for packaging Python dependencies",
)
@click.option(
    "--project-type",
    type=click.Choice(["python", "poetry"]),
    help="The type of project to create.",
    default="python",
)
def init(path, dockerfile, project_type):
    if dockerfile:
        click.echo("Creating sample Dockerfile...")
        PythonProject().copy_single_file("Dockerfile")
    else:
        kls = ProjectDetector().detect(project_type)
        click.echo("Initializing project")
        kls().initialize(path)


@click.command()
@click.option(
    "--entry-point",
    type=click.Path(exists=True, dir_okay=False, allow_dash=False),
    help="Entrypoint file",
    required=True,
)
@click.pass_obj
def package(project, entry_point):
    p = project(entry_point)
    p.build()


@click.command()
@click.option(
    "--entry-point",
    type=click.Path(exists=True, dir_okay=False, allow_dash=False),
    help="PySpark file to deploy",
    required=True,
)
@click.option(
    "--s3-code-uri",
    help="Where to copy code artifacts to",
    required=True,
)
@click.pass_obj
def deploy(project, entry_point, s3_code_uri):
    p = project(entry_point)
    p.deploy(s3_code_uri)


@click.command()
@click.option("--application-id", help="EMR Serverless Application ID", required=True)
@click.option(
    "--entry-point",
    type=click.Path(exists=True, dir_okay=False, allow_dash=False),
    help="Python or Jar file for the main entrypoint",
)
@click.option("--job-role", help="IAM Role ARN to use for the job execution")
@click.option("--wait", default=False, is_flag=True, help="Wait for job to finish")
@click.option("--s3-code-uri", help="Where to copy code artifacts to")
@click.option("--job-name", help="The name of the job", default="emr-cli job")
@click.option(
    "--job-args",
    help="Comma-delimited string of arguments to be passed to Spark job",
    default=None,
)
@click.option(
    "--spark-submit-opts",
    help="String of spark-submit options",
    default=None,
)
@click.option(
    "--build",
    help="Do not package and deploy the job assets",
    default=False,
    is_flag=True,
)
@click.pass_obj
def run(
    project,
    application_id,
    entry_point,
    job_role,
    wait,
    s3_code_uri,
    job_name,
    job_args,
    spark_submit_opts,
    build,
):
    # We require entry-point and s3-code-uri
    if entry_point is None or s3_code_uri is None:
        raise click.BadArgumentUsage(
            "--entry-point and --s3-code-uri are required if --build is used."
        )
    p = project(entry_point, s3_code_uri)

    if build:
        p.build()
        p.deploy(s3_code_uri)

    if application_id is not None:
        # We require entry-point and job-role
        if entry_point is None or job_role is None:
            raise click.BadArgumentUsage(
                "--entry-point and --job-role are required if --application-id is used."
            )

        if job_args:
            job_args = job_args.split(",")
        emrs = EMRServerless(application_id, job_role, p)
        emrs.run_job(job_name, job_args, spark_submit_opts, wait)


cli.add_command(package)
cli.add_command(deploy)
cli.add_command(run)
cli.add_command(init)
cli.add_command(bootstrap)

if __name__ == "__main__":
    cli()
